@@ -32,6 +32,26 @@ macro_rules! t {
     };
 }
 
+#[macro_export]
+macro_rules! curr_dir {
+    () => {
+        $crate::_curr_dir(std::path::Path::new(file!()));
+    };
+}
+
+#[doc(hidden)]
+pub fn _curr_dir(mut file_path: &'static Path) -> &'static Path {
+    if !file_path.exists() {
+        // HACK: Must be running in the rust-lang/rust workspace, adjust the paths accordingly.
+        let prefix = PathBuf::from("src").join("tools").join("cargo");
+        if let Ok(crate_relative) = file_path.strip_prefix(prefix) {
+            file_path = crate_relative
+        }
+    }
+    assert!(file_path.exists(), "{} does not exist", file_path.display());
+    file_path.parent().unwrap()
+}
+
 #[track_caller]
 pub fn panic_error(what: &str, err: impl Into<anyhow::Error>) -> ! {
     let err = err.into();
@@ -59,6 +79,13 @@ pub mod publish;
 pub mod registry;
 pub mod tools;
 
+pub mod prelude {
+    pub use crate::ArgLine;
+    pub use crate::CargoCommand;
+    pub use crate::ChannelChanger;
+    pub use crate::TestEnv;
+}
+
 /*
  *
  * ===== Builders =====
@@ -156,10 +183,16 @@ impl SymlinkBuilder {
     }
 }
 
+/// A cargo project to run tests against.
+///
+/// See [`ProjectBuilder`] or [`Project::from_template`] to get started.
 pub struct Project {
     root: PathBuf,
 }
 
+/// Create a project to run tests against
+///
+/// The project can be constructed programmatically or from the filesystem with [`Project::from_template`]
 #[must_use]
 pub struct ProjectBuilder {
     root: Project,
@@ -284,6 +317,14 @@ impl ProjectBuilder {
 }
 
 impl Project {
+    /// Copy the test project from a fixed state
+    pub fn from_template(template_path: impl AsRef<std::path::Path>) -> Self {
+        let root = paths::root();
+        let project_root = root.join("case");
+        snapbox::path::copy_template(template_path.as_ref(), &project_root).unwrap();
+        Self { root: project_root }
+    }
+
     /// Root of the project, ex: `/path/to/cargo/target/cit/t0/foo`
     pub fn root(&self) -> PathBuf {
         self.root.clone()
@@ -371,7 +412,7 @@ impl Project {
     pub fn cargo(&self, cmd: &str) -> Execs {
         let mut execs = self.process(&cargo_exe());
         if let Some(ref mut p) = execs.process_builder {
-            split_and_add_args(p, cmd);
+            p.arg_line(cmd);
         }
         execs
     }
@@ -470,24 +511,8 @@ pub fn main_file(println: &str, deps: &[&str]) -> String {
     buf
 }
 
-// Path to cargo executables
-pub fn cargo_dir() -> PathBuf {
-    env::var_os("CARGO_BIN_PATH")
-        .map(PathBuf::from)
-        .or_else(|| {
-            env::current_exe().ok().map(|mut path| {
-                path.pop();
-                if path.ends_with("deps") {
-                    path.pop();
-                }
-                path
-            })
-        })
-        .unwrap_or_else(|| panic!("CARGO_BIN_PATH wasn't set. Cannot continue running test"))
-}
-
 pub fn cargo_exe() -> PathBuf {
-    cargo_dir().join(format!("cargo{}", env::consts::EXE_SUFFIX))
+    snapbox::cmd::cargo_bin("cargo")
 }
 
 /// This is the raw output from the process.
@@ -1099,110 +1124,186 @@ pub fn process<T: AsRef<OsStr>>(t: T) -> ProcessBuilder {
 
 fn _process(t: &OsStr) -> ProcessBuilder {
     let mut p = ProcessBuilder::new(t);
-
-    // In general just clear out all cargo-specific configuration already in the
-    // environment. Our tests all assume a "default configuration" unless
-    // specified otherwise.
-    for (k, _v) in env::vars() {
-        if k.starts_with("CARGO_") {
-            p.env_remove(&k);
-        }
-    }
-    if env::var_os("RUSTUP_TOOLCHAIN").is_some() {
-        // Override the PATH to avoid executing the rustup wrapper thousands
-        // of times. This makes the testsuite run substantially faster.
-        lazy_static::lazy_static! {
-            static ref RUSTC_DIR: PathBuf = {
-                match ProcessBuilder::new("rustup")
-                    .args(&["which", "rustc"])
-                    .exec_with_output()
-                {
-                    Ok(output) => {
-                        let s = str::from_utf8(&output.stdout).expect("utf8").trim();
-                        let mut p = PathBuf::from(s);
-                        p.pop();
-                        p
-                    }
-                    Err(e) => {
-                        panic!("RUSTUP_TOOLCHAIN was set, but could not run rustup: {}", e);
-                    }
-                }
-            };
-        }
-        let path = env::var_os("PATH").unwrap_or_default();
-        let paths = env::split_paths(&path);
-        let new_path = env::join_paths(std::iter::once(RUSTC_DIR.clone()).chain(paths)).unwrap();
-        p.env("PATH", new_path);
-    }
-
-    p.cwd(&paths::root())
-        .env("HOME", paths::home())
-        .env("CARGO_HOME", paths::home().join(".cargo"))
-        .env("__CARGO_TEST_ROOT", paths::root())
-        // Force Cargo to think it's on the stable channel for all tests, this
-        // should hopefully not surprise us as we add cargo features over time and
-        // cargo rides the trains.
-        .env("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS", "stable")
-        // For now disable incremental by default as support hasn't ridden to the
-        // stable channel yet. Once incremental support hits the stable compiler we
-        // can switch this to one and then fix the tests.
-        .env("CARGO_INCREMENTAL", "0")
-        .env_remove("__CARGO_DEFAULT_LIB_METADATA")
-        .env_remove("RUSTC")
-        .env_remove("RUSTDOC")
-        .env_remove("RUSTC_WRAPPER")
-        .env_remove("RUSTFLAGS")
-        .env_remove("RUSTDOCFLAGS")
-        .env_remove("XDG_CONFIG_HOME") // see #2345
-        .env("GIT_CONFIG_NOSYSTEM", "1") // keep trying to sandbox ourselves
-        .env_remove("EMAIL")
-        .env_remove("USER") // not set on some rust-lang docker images
-        .env_remove("MFLAGS")
-        .env_remove("MAKEFLAGS")
-        .env_remove("GIT_AUTHOR_NAME")
-        .env_remove("GIT_AUTHOR_EMAIL")
-        .env_remove("GIT_COMMITTER_NAME")
-        .env_remove("GIT_COMMITTER_EMAIL")
-        .env_remove("MSYSTEM"); // assume cmd.exe everywhere on windows
-    if cfg!(target_os = "macos") {
-        // Work-around a bug in macOS 10.15, see `link_or_copy` for details.
-        p.env("__CARGO_COPY_DONT_LINK_DO_NOT_USE_THIS", "1");
-    }
+    p.cwd(&paths::root()).test_env();
     p
 }
 
-pub trait ChannelChanger: Sized {
-    fn masquerade_as_nightly_cargo(&mut self) -> &mut Self;
+/// Enable nightly features for testing
+pub trait ChannelChanger {
+    fn masquerade_as_nightly_cargo(self) -> Self;
 }
 
-impl ChannelChanger for ProcessBuilder {
-    fn masquerade_as_nightly_cargo(&mut self) -> &mut Self {
+impl ChannelChanger for &mut ProcessBuilder {
+    fn masquerade_as_nightly_cargo(self) -> Self {
         self.env("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS", "nightly")
     }
 }
 
-fn split_and_add_args(p: &mut ProcessBuilder, s: &str) {
-    for mut arg in s.split_whitespace() {
-        if (arg.starts_with('"') && arg.ends_with('"'))
-            || (arg.starts_with('\'') && arg.ends_with('\''))
-        {
-            arg = &arg[1..(arg.len() - 1).max(1)];
-        } else if arg.contains(&['"', '\''][..]) {
-            panic!("shell-style argument parsing is not supported")
+impl ChannelChanger for snapbox::cmd::Command {
+    fn masquerade_as_nightly_cargo(self) -> Self {
+        self.env("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS", "nightly")
+    }
+}
+
+/// Establish a process's test environment
+pub trait TestEnv: Sized {
+    fn test_env(mut self) -> Self {
+        // In general just clear out all cargo-specific configuration already in the
+        // environment. Our tests all assume a "default configuration" unless
+        // specified otherwise.
+        for (k, _v) in env::vars() {
+            if k.starts_with("CARGO_") {
+                self = self.env_remove(&k);
+            }
         }
-        p.arg(arg);
+        if env::var_os("RUSTUP_TOOLCHAIN").is_some() {
+            // Override the PATH to avoid executing the rustup wrapper thousands
+            // of times. This makes the testsuite run substantially faster.
+            lazy_static::lazy_static! {
+                static ref RUSTC_DIR: PathBuf = {
+                    match ProcessBuilder::new("rustup")
+                        .args(&["which", "rustc"])
+                        .exec_with_output()
+                    {
+                        Ok(output) => {
+                            let s = str::from_utf8(&output.stdout).expect("utf8").trim();
+                            let mut p = PathBuf::from(s);
+                            p.pop();
+                            p
+                        }
+                        Err(e) => {
+                            panic!("RUSTUP_TOOLCHAIN was set, but could not run rustup: {}", e);
+                        }
+                    }
+                };
+            }
+            let path = env::var_os("PATH").unwrap_or_default();
+            let paths = env::split_paths(&path);
+            let new_path =
+                env::join_paths(std::iter::once(RUSTC_DIR.clone()).chain(paths)).unwrap();
+            self = self.env("PATH", new_path);
+        }
+
+        self = self
+            .current_dir(&paths::root())
+            .env("HOME", paths::home())
+            .env("CARGO_HOME", paths::home().join(".cargo"))
+            .env("__CARGO_TEST_ROOT", paths::root())
+            // Force Cargo to think it's on the stable channel for all tests, this
+            // should hopefully not surprise us as we add cargo features over time and
+            // cargo rides the trains.
+            .env("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS", "stable")
+            // For now disable incremental by default as support hasn't ridden to the
+            // stable channel yet. Once incremental support hits the stable compiler we
+            // can switch this to one and then fix the tests.
+            .env("CARGO_INCREMENTAL", "0")
+            .env_remove("__CARGO_DEFAULT_LIB_METADATA")
+            .env_remove("RUSTC")
+            .env_remove("RUSTDOC")
+            .env_remove("RUSTC_WRAPPER")
+            .env_remove("RUSTFLAGS")
+            .env_remove("RUSTDOCFLAGS")
+            .env_remove("XDG_CONFIG_HOME") // see #2345
+            .env("GIT_CONFIG_NOSYSTEM", "1") // keep trying to sandbox ourselves
+            .env_remove("EMAIL")
+            .env_remove("USER") // not set on some rust-lang docker images
+            .env_remove("MFLAGS")
+            .env_remove("MAKEFLAGS")
+            .env_remove("GIT_AUTHOR_NAME")
+            .env_remove("GIT_AUTHOR_EMAIL")
+            .env_remove("GIT_COMMITTER_NAME")
+            .env_remove("GIT_COMMITTER_EMAIL")
+            .env_remove("MSYSTEM"); // assume cmd.exe everywhere on windows
+        if cfg!(target_os = "macos") {
+            // Work-around a bug in macOS 10.15, see `link_or_copy` for details.
+            self = self.env("__CARGO_COPY_DONT_LINK_DO_NOT_USE_THIS", "1");
+        }
+        self
+    }
+
+    fn current_dir<S: AsRef<std::path::Path>>(self, path: S) -> Self;
+    fn env<S: AsRef<std::ffi::OsStr>>(self, key: &str, value: S) -> Self;
+    fn env_remove(self, key: &str) -> Self;
+}
+
+impl TestEnv for &mut ProcessBuilder {
+    fn current_dir<S: AsRef<std::path::Path>>(self, path: S) -> Self {
+        let path = path.as_ref();
+        self.cwd(path)
+    }
+    fn env<S: AsRef<std::ffi::OsStr>>(self, key: &str, value: S) -> Self {
+        self.env(key, value)
+    }
+    fn env_remove(self, key: &str) -> Self {
+        self.env_remove(key)
+    }
+}
+
+impl TestEnv for snapbox::cmd::Command {
+    fn current_dir<S: AsRef<std::path::Path>>(self, path: S) -> Self {
+        self.current_dir(path)
+    }
+    fn env<S: AsRef<std::ffi::OsStr>>(self, key: &str, value: S) -> Self {
+        self.env(key, value)
+    }
+    fn env_remove(self, key: &str) -> Self {
+        self.env_remove(key)
+    }
+}
+
+/// Test the cargo command
+pub trait CargoCommand {
+    fn cargo() -> Self;
+}
+
+impl CargoCommand for snapbox::cmd::Command {
+    fn cargo() -> Self {
+        Self::new(cargo_exe())
+            .with_assert(compare::assert())
+            .test_env()
+    }
+}
+
+/// Add a list of arguments as a line
+pub trait ArgLine: Sized {
+    fn arg_line(mut self, s: &str) -> Self {
+        for mut arg in s.split_whitespace() {
+            if (arg.starts_with('"') && arg.ends_with('"'))
+                || (arg.starts_with('\'') && arg.ends_with('\''))
+            {
+                arg = &arg[1..(arg.len() - 1).max(1)];
+            } else if arg.contains(&['"', '\''][..]) {
+                panic!("shell-style argument parsing is not supported")
+            }
+            self = self.arg(arg);
+        }
+        self
+    }
+
+    fn arg<S: AsRef<std::ffi::OsStr>>(self, s: S) -> Self;
+}
+
+impl ArgLine for &mut ProcessBuilder {
+    fn arg<S: AsRef<std::ffi::OsStr>>(self, s: S) -> Self {
+        self.arg(s)
+    }
+}
+
+impl ArgLine for snapbox::cmd::Command {
+    fn arg<S: AsRef<std::ffi::OsStr>>(self, s: S) -> Self {
+        self.arg(s)
     }
 }
 
 pub fn cargo_process(s: &str) -> Execs {
     let mut p = process(&cargo_exe());
-    split_and_add_args(&mut p, s);
+    p.arg_line(s);
     execs().with_process_builder(p)
 }
 
 pub fn git_process(s: &str) -> ProcessBuilder {
     let mut p = process("git");
-    split_and_add_args(&mut p, s);
+    p.arg_line(s);
     p
 }
 
