@@ -14,6 +14,8 @@ pub struct BuildConfig {
     pub requested_kinds: Vec<CompileKind>,
     /// Number of rustc jobs to run in parallel.
     pub jobs: u32,
+    /// Do not abort the build as soon as there is an error.
+    pub keep_going: bool,
     /// Build profile
     pub requested_profile: InternedString,
     /// The mode we are compiling in.
@@ -39,6 +41,8 @@ pub struct BuildConfig {
     pub export_dir: Option<PathBuf>,
     /// `true` to output a future incompatibility report at the end of the build
     pub future_incompat_report: bool,
+    /// Which kinds of build timings to output (empty if none).
+    pub timing_outputs: Vec<TimingOutput>,
 }
 
 impl BuildConfig {
@@ -53,6 +57,7 @@ impl BuildConfig {
     pub fn new(
         config: &Config,
         jobs: Option<u32>,
+        keep_going: bool,
         requested_targets: &[String],
         mode: CompileMode,
     ) -> CargoResult<BuildConfig> {
@@ -69,10 +74,19 @@ impl BuildConfig {
             )?;
         }
         let jobs = jobs.or(cfg.jobs).unwrap_or(::num_cpus::get() as u32);
+        if jobs == 0 {
+            anyhow::bail!("jobs may not be 0");
+        }
+
+        if config.cli_unstable().build_std.is_some() && requested_kinds[0].is_host() {
+            // TODO: This should eventually be fixed.
+            anyhow::bail!("-Zbuild-std requires --target");
+        }
 
         Ok(BuildConfig {
             requested_kinds,
             jobs,
+            keep_going,
             requested_profile: InternedString::new("dev"),
             mode,
             message_format: MessageFormat::Human,
@@ -83,6 +97,7 @@ impl BuildConfig {
             rustfix_diagnostic_server: RefCell::new(None),
             export_dir: None,
             future_incompat_report: false,
+            timing_outputs: Vec::new(),
         })
     }
 
@@ -146,6 +161,8 @@ pub enum CompileMode {
     Doc { deps: bool },
     /// A target that will be tested with `rustdoc`.
     Doctest,
+    /// An example or library that will be scraped for function calls by `rustdoc`.
+    Docscrape,
     /// A marker for Units that represent the execution of a `build.rs` script.
     RunCustomBuild,
 }
@@ -163,6 +180,7 @@ impl ser::Serialize for CompileMode {
             Bench => "bench".serialize(s),
             Doc { .. } => "doc".serialize(s),
             Doctest => "doctest".serialize(s),
+            Docscrape => "docscrape".serialize(s),
             RunCustomBuild => "run-custom-build".serialize(s),
         }
     }
@@ -184,6 +202,11 @@ impl CompileMode {
         self == CompileMode::Doctest
     }
 
+    /// Returns `true` if this is scraping examples for documentation.
+    pub fn is_doc_scrape(self) -> bool {
+        self == CompileMode::Docscrape
+    }
+
     /// Returns `true` if this is any type of test (test, benchmark, doc test, or
     /// check test).
     pub fn is_any_test(self) -> bool {
@@ -208,4 +231,24 @@ impl CompileMode {
     pub fn is_run_custom_build(self) -> bool {
         self == CompileMode::RunCustomBuild
     }
+
+    /// Returns `true` if this mode may generate an executable.
+    ///
+    /// Note that this also returns `true` for building libraries, so you also
+    /// have to check the target.
+    pub fn generates_executable(self) -> bool {
+        matches!(
+            self,
+            CompileMode::Test | CompileMode::Bench | CompileMode::Build
+        )
+    }
+}
+
+/// Kinds of build timings we can output.
+#[derive(Clone, Copy, PartialEq, Debug, Eq, Hash, PartialOrd, Ord)]
+pub enum TimingOutput {
+    /// Human-readable HTML report
+    Html,
+    /// Machine-readable JSON (unstable)
+    Json,
 }
