@@ -1,6 +1,7 @@
 use std::collections::HashMap;
 use std::fmt::{self, Debug, Formatter};
 use std::path::{Path, PathBuf};
+use std::task::Poll;
 
 use crate::core::source::MaybePackage;
 use crate::core::{Dependency, Package, PackageId, Source, SourceId, Summary};
@@ -17,6 +18,7 @@ pub struct DirectorySource<'cfg> {
     root: PathBuf,
     packages: HashMap<PackageId, (Package, Checksum)>,
     config: &'cfg Config,
+    updated: bool,
 }
 
 #[derive(Deserialize)]
@@ -32,6 +34,7 @@ impl<'cfg> DirectorySource<'cfg> {
             root: path.to_path_buf(),
             config,
             packages: HashMap::new(),
+            updated: false,
         }
     }
 }
@@ -43,21 +46,31 @@ impl<'cfg> Debug for DirectorySource<'cfg> {
 }
 
 impl<'cfg> Source for DirectorySource<'cfg> {
-    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> Poll<CargoResult<()>> {
+        if !self.updated {
+            return Poll::Pending;
+        }
         let packages = self.packages.values().map(|p| &p.0);
         let matches = packages.filter(|pkg| dep.matches(pkg.summary()));
         for summary in matches.map(|pkg| pkg.summary().clone()) {
             f(summary);
         }
-        Ok(())
+        Poll::Ready(Ok(()))
     }
 
-    fn fuzzy_query(&mut self, _dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+    fn fuzzy_query(
+        &mut self,
+        _dep: &Dependency,
+        f: &mut dyn FnMut(Summary),
+    ) -> Poll<CargoResult<()>> {
+        if !self.updated {
+            return Poll::Pending;
+        }
         let packages = self.packages.values().map(|p| &p.0);
         for summary in packages.map(|pkg| pkg.summary().clone()) {
             f(summary);
         }
-        Ok(())
+        Poll::Ready(Ok(()))
     }
 
     fn supports_checksums(&self) -> bool {
@@ -72,7 +85,10 @@ impl<'cfg> Source for DirectorySource<'cfg> {
         self.source_id
     }
 
-    fn update(&mut self) -> CargoResult<()> {
+    fn block_until_ready(&mut self) -> CargoResult<()> {
+        if self.updated {
+            return Ok(());
+        }
         self.packages.clear();
         let entries = self.root.read_dir().with_context(|| {
             format!(
@@ -143,6 +159,7 @@ impl<'cfg> Source for DirectorySource<'cfg> {
             self.packages.insert(pkg.package_id(), (pkg, cksum));
         }
 
+        self.updated = true;
         Ok(())
     }
 
@@ -205,4 +222,8 @@ impl<'cfg> Source for DirectorySource<'cfg> {
     fn is_yanked(&mut self, _pkg: PackageId) -> CargoResult<bool> {
         Ok(false)
     }
+
+    fn invalidate_cache(&mut self) {
+        // Path source has no local cache.
+    }
 }
