@@ -1,6 +1,7 @@
+use std::collections::HashSet;
 use std::fmt::{self, Debug, Formatter};
-use std::fs;
 use std::path::{Path, PathBuf};
+use std::task::Poll;
 
 use crate::core::source::MaybePackage;
 use crate::core::{Dependency, Package, PackageId, Source, SourceId, Summary};
@@ -10,8 +11,8 @@ use anyhow::Context as _;
 use cargo_util::paths;
 use filetime::FileTime;
 use ignore::gitignore::GitignoreBuilder;
-use ignore::Match;
 use log::{trace, warn};
+use walkdir::WalkDir;
 
 pub struct PathSource<'cfg> {
     source_id: SourceId,
@@ -130,39 +131,36 @@ impl<'cfg> PathSource<'cfg> {
         }
         let ignore_include = include_builder.build()?;
 
-        let ignore_should_package = |relative_path: &Path, is_dir: bool| -> CargoResult<bool> {
+        let ignore_should_package = |relative_path: &Path, is_dir: bool| {
             // "Include" and "exclude" options are mutually exclusive.
             if no_include_option {
-                match ignore_exclude.matched_path_or_any_parents(relative_path, is_dir) {
-                    Match::None => Ok(true),
-                    Match::Ignore(_) => Ok(false),
-                    Match::Whitelist(_) => Ok(true),
-                }
+                !ignore_exclude
+                    .matched_path_or_any_parents(relative_path, is_dir)
+                    .is_ignore()
             } else {
                 if is_dir {
                     // Generally, include directives don't list every
                     // directory (nor should they!). Just skip all directory
                     // checks, and only check files.
-                    return Ok(true);
+                    return true;
                 }
-                match ignore_include
+                ignore_include
                     .matched_path_or_any_parents(relative_path, /* is_dir */ false)
-                {
-                    Match::None => Ok(false),
-                    Match::Ignore(_) => Ok(true),
-                    Match::Whitelist(_) => Ok(false),
-                }
+                    .is_ignore()
             }
         };
 
-        let mut filter = |path: &Path, is_dir: bool| -> CargoResult<bool> {
-            let relative_path = path.strip_prefix(root)?;
+        let mut filter = |path: &Path, is_dir: bool| {
+            let relative_path = match path.strip_prefix(root) {
+                Ok(p) => p,
+                Err(_) => return false,
+            };
 
             let rel = relative_path.as_os_str();
             if rel == "Cargo.lock" {
-                return Ok(pkg.include_lockfile());
+                return pkg.include_lockfile();
             } else if rel == "Cargo.toml" {
-                return Ok(true);
+                return true;
             }
 
             ignore_should_package(relative_path, is_dir)
@@ -224,7 +222,7 @@ impl<'cfg> PathSource<'cfg> {
         &self,
         pkg: &Package,
         repo: &git2::Repository,
-        filter: &mut dyn FnMut(&Path, bool) -> CargoResult<bool>,
+        filter: &mut dyn FnMut(&Path, bool) -> bool,
     ) -> CargoResult<Vec<PathBuf>> {
         warn!("list_files_git {}", pkg.package_id());
         let index = repo.index()?;
@@ -261,19 +259,36 @@ impl<'cfg> PathSource<'cfg> {
             opts.pathspec(suffix);
         }
         let statuses = repo.statuses(Some(&mut opts))?;
-        let untracked = statuses.iter().filter_map(|entry| match entry.status() {
-            // Don't include Cargo.lock if it is untracked. Packaging will
-            // generate a new one as needed.
-            git2::Status::WT_NEW if entry.path() != Some("Cargo.lock") => {
-                Some((join(root, entry.path_bytes()), None))
-            }
-            _ => None,
-        });
+        let mut skip_paths = HashSet::new();
+        let untracked: Vec<_> = statuses
+            .iter()
+            .filter_map(|entry| {
+                match entry.status() {
+                    // Don't include Cargo.lock if it is untracked. Packaging will
+                    // generate a new one as needed.
+                    git2::Status::WT_NEW if entry.path() != Some("Cargo.lock") => {
+                        Some(Ok((join(root, entry.path_bytes()), None)))
+                    }
+                    git2::Status::WT_DELETED => {
+                        let path = match join(root, entry.path_bytes()) {
+                            Ok(p) => p,
+                            Err(e) => return Some(Err(e)),
+                        };
+                        skip_paths.insert(path);
+                        None
+                    }
+                    _ => None,
+                }
+            })
+            .collect::<CargoResult<_>>()?;
 
         let mut subpackages_found = Vec::new();
 
         for (file_path, is_dir) in index_files.chain(untracked) {
             let file_path = file_path?;
+            if skip_paths.contains(&file_path) {
+                continue;
+            }
 
             // Filter out files blatantly outside this package. This is helped a
             // bit above via the `pathspec` function call, but we need to filter
@@ -326,10 +341,10 @@ impl<'cfg> PathSource<'cfg> {
                         ret.extend(files.into_iter());
                     }
                     Err(..) => {
-                        PathSource::walk(&file_path, &mut ret, false, filter)?;
+                        self.walk(&file_path, &mut ret, false, filter)?;
                     }
                 }
-            } else if (*filter)(&file_path, is_dir)? {
+            } else if filter(&file_path, is_dir) {
                 assert!(!is_dir);
                 // We found a file!
                 warn!("  found {}", file_path.display());
@@ -361,50 +376,71 @@ impl<'cfg> PathSource<'cfg> {
     fn list_files_walk(
         &self,
         pkg: &Package,
-        filter: &mut dyn FnMut(&Path, bool) -> CargoResult<bool>,
+        filter: &mut dyn FnMut(&Path, bool) -> bool,
     ) -> CargoResult<Vec<PathBuf>> {
         let mut ret = Vec::new();
-        PathSource::walk(pkg.root(), &mut ret, true, filter)?;
+        self.walk(pkg.root(), &mut ret, true, filter)?;
         Ok(ret)
     }
 
     fn walk(
+        &self,
         path: &Path,
         ret: &mut Vec<PathBuf>,
         is_root: bool,
-        filter: &mut dyn FnMut(&Path, bool) -> CargoResult<bool>,
+        filter: &mut dyn FnMut(&Path, bool) -> bool,
     ) -> CargoResult<()> {
-        let is_dir = path.is_dir();
-        if !is_root && !(*filter)(path, is_dir)? {
-            return Ok(());
-        }
-        if !is_dir {
-            ret.push(path.to_path_buf());
-            return Ok(());
-        }
-        // Don't recurse into any sub-packages that we have.
-        if !is_root && path.join("Cargo.toml").exists() {
-            return Ok(());
+        let walkdir = WalkDir::new(path)
+            .follow_links(true)
+            .into_iter()
+            .filter_entry(|entry| {
+                let path = entry.path();
+                let at_root = is_root && entry.depth() == 0;
+                let is_dir = entry.file_type().is_dir();
+
+                if !at_root && !filter(path, is_dir) {
+                    return false;
+                }
+
+                if !is_dir {
+                    return true;
+                }
+
+                // Don't recurse into any sub-packages that we have.
+                if !at_root && path.join("Cargo.toml").exists() {
+                    return false;
+                }
+
+                // Skip root Cargo artifacts.
+                if is_root
+                    && entry.depth() == 1
+                    && path.file_name().and_then(|s| s.to_str()) == Some("target")
+                {
+                    return false;
+                }
+
+                true
+            });
+        for entry in walkdir {
+            match entry {
+                Ok(entry) => {
+                    if !entry.file_type().is_dir() {
+                        ret.push(entry.into_path());
+                    }
+                }
+                Err(err) if err.loop_ancestor().is_some() => {
+                    self.config.shell().warn(err)?;
+                }
+                Err(err) => match err.path() {
+                    // If the error occurs with a path, simply recover from it.
+                    // Don't worry about error skipping here, the callers would
+                    // still hit the IO error if they do access it thereafter.
+                    Some(path) => ret.push(path.to_path_buf()),
+                    None => return Err(err.into()),
+                },
+            }
         }
 
-        // For package integration tests, we need to sort the paths in a deterministic order to
-        // be able to match stdout warnings in the same order.
-        //
-        // TODO: drop `collect` and sort after transition period and dropping warning tests.
-        // See rust-lang/cargo#4268 and rust-lang/cargo#4270.
-        let mut entries: Vec<PathBuf> = fs::read_dir(path)
-            .with_context(|| format!("cannot read {:?}", path))?
-            .map(|e| e.unwrap().path())
-            .collect();
-        entries.sort_unstable_by(|a, b| a.as_os_str().cmp(b.as_os_str()));
-        for path in entries {
-            let name = path.file_name().and_then(|s| s.to_str());
-            if is_root && name == Some("target") {
-                // Skip Cargo artifacts.
-                continue;
-            }
-            PathSource::walk(&path, ret, false, filter)?;
-        }
         Ok(())
     }
 
@@ -442,6 +478,16 @@ impl<'cfg> PathSource<'cfg> {
     pub fn path(&self) -> &Path {
         &self.path
     }
+
+    pub fn update(&mut self) -> CargoResult<()> {
+        if !self.updated {
+            let packages = self.read_packages()?;
+            self.packages.extend(packages.into_iter());
+            self.updated = true;
+        }
+
+        Ok(())
+    }
 }
 
 impl<'cfg> Debug for PathSource<'cfg> {
@@ -451,20 +497,26 @@ impl<'cfg> Debug for PathSource<'cfg> {
 }
 
 impl<'cfg> Source for PathSource<'cfg> {
-    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> Poll<CargoResult<()>> {
+        self.update()?;
         for s in self.packages.iter().map(|p| p.summary()) {
             if dep.matches(s) {
                 f(s.clone())
             }
         }
-        Ok(())
+        Poll::Ready(Ok(()))
     }
 
-    fn fuzzy_query(&mut self, _dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+    fn fuzzy_query(
+        &mut self,
+        _dep: &Dependency,
+        f: &mut dyn FnMut(Summary),
+    ) -> Poll<CargoResult<()>> {
+        self.update()?;
         for s in self.packages.iter().map(|p| p.summary()) {
             f(s.clone())
         }
-        Ok(())
+        Poll::Ready(Ok(()))
     }
 
     fn supports_checksums(&self) -> bool {
@@ -479,19 +531,9 @@ impl<'cfg> Source for PathSource<'cfg> {
         self.source_id
     }
 
-    fn update(&mut self) -> CargoResult<()> {
-        if !self.updated {
-            let packages = self.read_packages()?;
-            self.packages.extend(packages.into_iter());
-            self.updated = true;
-        }
-
-        Ok(())
-    }
-
     fn download(&mut self, id: PackageId) -> CargoResult<MaybePackage> {
         trace!("getting packages; id={}", id);
-
+        self.update()?;
         let pkg = self.packages.iter().find(|pkg| pkg.package_id() == id);
         pkg.cloned()
             .map(MaybePackage::Ready)
@@ -523,4 +565,12 @@ impl<'cfg> Source for PathSource<'cfg> {
     fn is_yanked(&mut self, _pkg: PackageId) -> CargoResult<bool> {
         Ok(false)
     }
+
+    fn block_until_ready(&mut self) -> CargoResult<()> {
+        self.update()
+    }
+
+    fn invalidate_cache(&mut self) {
+        // Path source has no local cache.
+    }
 }
