@@ -647,7 +647,7 @@ Caused by:
         "owner",
         "publish",
         "search",
-        "yank --vers 0.0.1",
+        "yank --version 0.0.1",
     ] {
         p.cargo(cmd)
             .arg("--registry")
@@ -726,12 +726,12 @@ fn no_api() {
         .with_stderr_contains(&err)
         .run();
 
-    p.cargo("yank --registry alternative --vers=0.0.1 bar")
+    p.cargo("yank --registry alternative --version=0.0.1 bar")
         .with_status(101)
         .with_stderr_contains(&err)
         .run();
 
-    p.cargo("yank --registry alternative --vers=0.0.1 bar")
+    p.cargo("yank --registry alternative --version=0.0.1 bar")
         .with_stderr_contains(&err)
         .with_status(101)
         .run();
@@ -823,6 +823,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -880,6 +881,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -905,6 +907,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -930,6 +933,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -980,6 +984,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -1018,6 +1023,7 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "rust_version": null,
                         "homepage": null,
                         "documentation": null,
                         "edition": "2015",
@@ -1115,6 +1121,7 @@ fn unknown_registry() {
                   "keywords": [],
                   "readme": null,
                   "repository": null,
+                  "rust_version": null,
                   "homepage": null,
                   "documentation": null,
                   "edition": "2015",
@@ -1140,6 +1147,7 @@ fn unknown_registry() {
                   "keywords": [],
                   "readme": null,
                   "repository": null,
+                  "rust_version": null,
                   "homepage": null,
                   "documentation": null,
                   "edition": "2015",
@@ -1178,6 +1186,7 @@ fn unknown_registry() {
                   "keywords": [],
                   "readme": null,
                   "repository": null,
+                  "rust_version": null,
                   "homepage": null,
                   "documentation": null,
                   "edition": "2015",
@@ -1302,7 +1311,7 @@ Caused by:
 #[cargo_test]
 fn both_index_and_registry() {
     let p = project().file("src/lib.rs", "").build();
-    for cmd in &["publish", "owner", "search", "yank --vers 1.0.0"] {
+    for cmd in &["publish", "owner", "search", "yank --version 1.0.0"] {
         p.cargo(cmd)
             .arg("--registry=foo")
             .arg("--index=foo")
