@@ -5,6 +5,7 @@ use cargo_test_support::paths;
 use cargo_test_support::registry::{self, Package};
 use cargo_test_support::{basic_manifest, project};
 use std::fs;
+use toml_edit::easy as toml;
 
 #[cargo_test]
 fn replace() {
@@ -66,50 +67,6 @@ fn replace() {
     p.cargo("build").with_stderr("[FINISHED] [..]").run();
 }
 
-#[cargo_test]
-fn from_config_without_z() {
-    Package::new("bar", "0.1.0").publish();
-
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-                [package]
-                name = "foo"
-                version = "0.0.1"
-                authors = []
-
-                [dependencies]
-                bar = "0.1.0"
-            "#,
-        )
-        .file(
-            ".cargo/config.toml",
-            r#"
-                [patch.crates-io]
-                bar = { path = 'bar' }
-            "#,
-        )
-        .file("src/lib.rs", "")
-        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.1"))
-        .file("bar/src/lib.rs", r#""#)
-        .build();
-
-    p.cargo("build")
-        .with_stderr(
-            "\
-[WARNING] `[patch]` in cargo config was ignored, the -Zpatch-in-config command-line flag is required
-[UPDATING] `dummy-registry` index
-[DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.1.0 ([..])
-[COMPILING] bar v0.1.0
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
-",
-        )
-        .run();
-}
-
 #[cargo_test]
 fn from_config() {
     Package::new("bar", "0.1.0").publish();
@@ -139,8 +96,7 @@ fn from_config() {
         .file("bar/src/lib.rs", r#""#)
         .build();
 
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [UPDATING] `dummy-registry` index
@@ -181,8 +137,7 @@ fn from_config_relative() {
         .file("bar/src/lib.rs", r#""#)
         .build();
 
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [UPDATING] `dummy-registry` index
@@ -211,14 +166,14 @@ fn from_config_precedence() {
                 bar = "0.1.0"
 
                 [patch.crates-io]
-                bar = { path = 'bar' }
+                bar = { path = 'no-such-path' }
             "#,
         )
         .file(
             ".cargo/config.toml",
             r#"
                 [patch.crates-io]
-                bar = { path = 'no-such-path' }
+                bar = { path = 'bar' }
             "#,
         )
         .file("src/lib.rs", "")
@@ -226,8 +181,7 @@ fn from_config_precedence() {
         .file("bar/src/lib.rs", r#""#)
         .build();
 
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [UPDATING] `dummy-registry` index
@@ -405,10 +359,10 @@ fn unused() {
             "\
 [UPDATING] `dummy-registry` index
 [WARNING] Patch `bar v0.2.0 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [DOWNLOADING] crates ...
 [DOWNLOADED] bar v0.1.0 [..]
 [COMPILING] bar v0.1.0
@@ -421,10 +375,10 @@ fn unused() {
         .with_stderr(
             "\
 [WARNING] Patch `bar v0.2.0 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [FINISHED] [..]
 ",
         )
@@ -441,6 +395,60 @@ fn unused() {
     );
 }
 
+#[cargo_test]
+fn unused_with_mismatch_source_being_patched() {
+    registry::alt_init();
+    Package::new("bar", "0.1.0").publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+
+                [dependencies]
+                bar = "0.1.0"
+
+                [patch.alternative]
+                bar = { path = "bar" }
+
+                [patch.crates-io]
+                bar = { path = "baz" }
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.2.0"))
+        .file("bar/src/lib.rs", "not rust code")
+        .file("baz/Cargo.toml", &basic_manifest("bar", "0.3.0"))
+        .file("baz/src/lib.rs", "not rust code")
+        .build();
+
+    p.cargo("build")
+        .with_stderr(
+            "\
+[UPDATING] `dummy-registry` index
+[WARNING] Patch `bar v0.2.0 ([CWD]/bar)` was not used in the crate graph.
+Perhaps you misspell the source URL being patched.
+Possible URLs for `[patch.<URL>]`:
+    crates-io
+[WARNING] Patch `bar v0.3.0 ([CWD]/baz)` was not used in the crate graph.
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.1.0 [..]
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+",
+        )
+        .run();
+}
+
 #[cargo_test]
 fn prefer_patch_version() {
     Package::new("bar", "0.1.2").publish();
@@ -519,16 +527,15 @@ fn unused_from_config() {
         .file("bar/src/lib.rs", "not rust code")
         .build();
 
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [UPDATING] `dummy-registry` index
 [WARNING] Patch `bar v0.2.0 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [DOWNLOADING] crates ...
 [DOWNLOADED] bar v0.1.0 [..]
 [COMPILING] bar v0.1.0
@@ -537,15 +544,14 @@ fn unused_from_config() {
 ",
         )
         .run();
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [WARNING] Patch `bar v0.2.0 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [FINISHED] [..]
 ",
         )
@@ -599,10 +605,10 @@ fn unused_git() {
 [UPDATING] git repository `file://[..]`
 [UPDATING] `dummy-registry` index
 [WARNING] Patch `bar v0.2.0 ([..])` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [DOWNLOADING] crates ...
 [DOWNLOADED] bar v0.1.0 [..]
 [COMPILING] bar v0.1.0
@@ -615,10 +621,10 @@ fn unused_git() {
         .with_stderr(
             "\
 [WARNING] Patch `bar v0.2.0 ([..])` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [FINISHED] [..]
 ",
         )
@@ -733,8 +739,7 @@ fn add_patch_from_config() {
         "#,
     );
 
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build")
         .with_stderr(
             "\
 [COMPILING] bar v0.1.0 ([CWD]/bar)
@@ -743,10 +748,7 @@ fn add_patch_from_config() {
 ",
         )
         .run();
-    p.cargo("build -Zpatch-in-config")
-        .masquerade_as_nightly_cargo()
-        .with_stderr("[FINISHED] [..]")
-        .run();
+    p.cargo("build").with_stderr("[FINISHED] [..]").run();
 }
 
 #[cargo_test]
@@ -805,10 +807,10 @@ fn add_ignored_patch() {
         .with_stderr(
             "\
 [WARNING] Patch `bar v0.1.1 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]",
         )
         .run();
@@ -816,10 +818,10 @@ fn add_ignored_patch() {
         .with_stderr(
             "\
 [WARNING] Patch `bar v0.1.1 ([CWD]/bar)` was not used in the crate graph.
-[..]
-[..]
-[..]
-[..]
+Check that [..]
+with the [..]
+what is [..]
+version. [..]
 [FINISHED] [..]",
         )
         .run();
@@ -1618,10 +1620,10 @@ fn cycle() {
         .with_stderr(
             "\
 [UPDATING] [..]
-error: cyclic package dependency: [..]
+[ERROR] cyclic package dependency: [..]
 package `[..]`
-    ... which is depended on by `[..]`
-    ... which is depended on by `[..]`
+    ... which satisfies dependency `[..]` of package `[..]`
+    ... which satisfies dependency `[..]` of package `[..]`
 ",
         )
         .run();
@@ -1767,10 +1769,9 @@ fn two_semver_compatible() {
         .with_stderr(
             "\
 warning: Patch `bar v0.1.1 [..]` was not used in the crate graph.
-Check that [..]
-with the [..]
-what is [..]
-version. [..]
+Perhaps you misspell the source URL being patched.
+Possible URLs for `[patch.<URL>]`:
+    [CWD]/bar
 [FINISHED] [..]",
         )
         .run();
@@ -1822,10 +1823,9 @@ fn multipatch_select_big() {
         .with_stderr(
             "\
 warning: Patch `bar v0.1.0 [..]` was not used in the crate graph.
-Check that [..]
-with the [..]
-what is [..]
-version. [..]
+Perhaps you misspell the source URL being patched.
+Possible URLs for `[patch.<URL>]`:
+    [CWD]/bar
 [FINISHED] [..]",
         )
         .run();
