@@ -186,7 +186,7 @@ fn lock_file_and_workspace() {
 #[cargo_test]
 fn note_resolve_changes() {
     // `multi` has multiple sources (path and registry).
-    Package::new("mutli", "0.1.0").publish();
+    Package::new("multi", "0.1.0").publish();
     // `updated` is always from registry, but should not change.
     Package::new("updated", "1.0.0").publish();
     // `patched` is [patch]ed.
@@ -200,7 +200,7 @@ fn note_resolve_changes() {
                 "0.0.1",
                 r#"
                 [dependencies]
-                mutli = { path = "mutli", version = "0.1" }
+                multi = { path = "multi", version = "0.1" }
                 updated = "1.0"
                 patched = "1.0"
 
@@ -210,8 +210,8 @@ fn note_resolve_changes() {
             ),
         )
         .file("src/main.rs", "fn main() {}")
-        .file("mutli/Cargo.toml", &basic_manifest("mutli", "0.1.0"))
-        .file("mutli/src/lib.rs", "")
+        .file("multi/Cargo.toml", &basic_manifest("multi", "0.1.0"))
+        .file("multi/src/lib.rs", "")
         .file("patched/Cargo.toml", &basic_manifest("patched", "1.0.0"))
         .file("patched/src/lib.rs", "")
         .build();
@@ -230,7 +230,7 @@ fn note_resolve_changes() {
 [ARCHIVING] Cargo.toml.orig
 [ARCHIVING] src/main.rs
 [UPDATING] `[..]` index
-[NOTE] package `mutli v0.1.0` added to the packaged Cargo.lock file, was originally sourced from `[..]/foo/mutli`
+[NOTE] package `multi v0.1.0` added to the packaged Cargo.lock file, was originally sourced from `[..]/foo/multi`
 [NOTE] package `patched v1.0.0` added to the packaged Cargo.lock file, was originally sourced from `[..]/foo/patched`
 ",
         )
