@@ -1,7 +1,7 @@
 //! Tests for the `cargo run` command.
 
-use cargo::util::paths::dylib_path_envvar;
 use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, project, Project};
+use cargo_util::paths::dylib_path_envvar;
 
 #[cargo_test]
 fn simple() {
@@ -22,18 +22,21 @@ fn simple() {
 }
 
 #[cargo_test]
-fn simple_quiet() {
+fn quiet_arg() {
     let p = project()
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    p.cargo("run -q").with_stdout("hello").run();
+    p.cargo("run -q").with_stderr("").with_stdout("hello").run();
 
-    p.cargo("run --quiet").with_stdout("hello").run();
+    p.cargo("run --quiet")
+        .with_stderr("")
+        .with_stdout("hello")
+        .run();
 }
 
 #[cargo_test]
-fn simple_quiet_and_verbose() {
+fn quiet_arg_and_verbose_arg() {
     let p = project()
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
@@ -45,19 +48,105 @@ fn simple_quiet_and_verbose() {
 }
 
 #[cargo_test]
-fn quiet_and_verbose_config() {
+fn quiet_arg_and_verbose_config() {
     let p = project()
         .file(
             ".cargo/config",
             r#"
-            [term]
-            verbose = true
-        "#,
+                [term]
+                verbose = true
+            "#,
         )
         .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
         .build();
 
-    p.cargo("run -q").run();
+    p.cargo("run -q").with_stderr("").with_stdout("hello").run();
+}
+
+#[cargo_test]
+fn verbose_arg_and_quiet_config() {
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+                [term]
+                quiet = true
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .build();
+
+    p.cargo("run -v")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
+[RUNNING] `rustc [..]
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/foo[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
+}
+
+#[cargo_test]
+fn quiet_config_alone() {
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+                [term]
+                quiet = true
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .build();
+
+    p.cargo("run").with_stderr("").with_stdout("hello").run();
+}
+
+#[cargo_test]
+fn verbose_config_alone() {
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+                [term]
+                verbose = true
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .build();
+
+    p.cargo("run")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
+[RUNNING] `rustc [..]
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/foo[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
+}
+
+#[cargo_test]
+fn quiet_config_and_verbose_config() {
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+                [term]
+                verbose = true
+                quiet = true
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .build();
+
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr("[ERROR] cannot set both `term.verbose` and `term.quiet`")
+        .run();
 }
 
 #[cargo_test]
@@ -66,11 +155,11 @@ fn simple_with_args() {
         .file(
             "src/main.rs",
             r#"
-            fn main() {
-                assert_eq!(std::env::args().nth(1).unwrap(), "hello");
-                assert_eq!(std::env::args().nth(2).unwrap(), "world");
-            }
-        "#,
+                fn main() {
+                    assert_eq!(std::env::args().nth(1).unwrap(), "hello");
+                    assert_eq!(std::env::args().nth(2).unwrap(), "world");
+                }
+            "#,
         )
         .build();
 
@@ -86,14 +175,14 @@ fn simple_with_non_utf8_args() {
         .file(
             "src/main.rs",
             r#"
-            use std::ffi::OsStr;
-            use std::os::unix::ffi::OsStrExt;
+                use std::ffi::OsStr;
+                use std::os::unix::ffi::OsStrExt;
 
-            fn main() {
-                assert_eq!(std::env::args_os().nth(1).unwrap(), OsStr::from_bytes(b"hello"));
-                assert_eq!(std::env::args_os().nth(2).unwrap(), OsStr::from_bytes(b"ab\xffcd"));
-            }
-        "#,
+                fn main() {
+                    assert_eq!(std::env::args_os().nth(1).unwrap(), OsStr::from_bytes(b"hello"));
+                    assert_eq!(std::env::args_os().nth(2).unwrap(), OsStr::from_bytes(b"ab\xffcd"));
+                }
+            "#,
         )
         .build();
 
@@ -118,7 +207,7 @@ fn exit_code() {
     );
     if !cfg!(unix) {
         output.push_str(
-            "[ERROR] process didn't exit successfully: `target[..]foo[..]` (exit code: 2)",
+            "[ERROR] process didn't exit successfully: `target[..]foo[..]` (exit [..]: 2)",
         );
     }
     p.cargo("run").with_status(2).with_stderr(output).run();
@@ -140,7 +229,7 @@ fn exit_code_verbose() {
     );
     if !cfg!(unix) {
         output.push_str(
-            "[ERROR] process didn't exit successfully: `target[..]foo[..]` (exit code: 2)",
+            "[ERROR] process didn't exit successfully: `target[..]foo[..]` (exit [..]: 2)",
         );
     }
 
@@ -187,18 +276,18 @@ fn specify_name() {
         .file(
             "src/bin/a.rs",
             r#"
-            #[allow(unused_extern_crates)]
-            extern crate foo;
-            fn main() { println!("hello a.rs"); }
-        "#,
+                #[allow(unused_extern_crates)]
+                extern crate foo;
+                fn main() { println!("hello a.rs"); }
+            "#,
         )
         .file(
             "src/bin/b.rs",
             r#"
-            #[allow(unused_extern_crates)]
-            extern crate foo;
-            fn main() { println!("hello b.rs"); }
-        "#,
+                #[allow(unused_extern_crates)]
+                extern crate foo;
+                fn main() { println!("hello b.rs"); }
+            "#,
         )
         .build();
 
@@ -232,12 +321,12 @@ fn specify_default_run() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            default-run = "a"
-        "#,
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                default-run = "a"
+            "#,
         )
         .file("src/lib.rs", "")
         .file("src/bin/a.rs", r#"fn main() { println!("hello A"); }"#)
@@ -255,12 +344,12 @@ fn bogus_default_run() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            default-run = "b"
-        "#,
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                default-run = "b"
+            "#,
         )
         .file("src/lib.rs", "")
         .file("src/bin/a.rs", r#"fn main() { println!("hello A"); }"#)
@@ -306,14 +395,14 @@ fn run_library_example() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            [[example]]
-            name = "bar"
-            crate_type = ["lib"]
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                [[example]]
+                name = "bar"
+                crate_type = ["lib"]
+            "#,
         )
         .file("src/lib.rs", "")
         .file("examples/bar.rs", "fn foo() {}")
@@ -331,13 +420,13 @@ fn run_bin_example() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            [[example]]
-            name = "bar"
-            crate_type = ["bin"]
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                [[example]]
+                name = "bar"
+                crate_type = ["bin"]
+            "#,
         )
         .file("src/lib.rs", "")
         .file("examples/bar.rs", r#"fn main() { println!("example"); }"#)
@@ -502,9 +591,9 @@ fn run_with_filename() {
         .file(
             "src/bin/a.rs",
             r#"
-            extern crate foo;
-            fn main() { println!("hello a.rs"); }
-        "#,
+                extern crate foo;
+                fn main() { println!("hello a.rs"); }
+            "#,
         )
         .file("examples/a.rs", r#"fn main() { println!("example"); }"#)
         .build();
@@ -586,43 +675,43 @@ fn example_with_release_flag() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "*"
-            path = "bar"
-        "#,
+                [dependencies.bar]
+                version = "*"
+                path = "bar"
+            "#,
         )
         .file(
             "examples/a.rs",
             r#"
-            extern crate bar;
+                extern crate bar;
 
-            fn main() {
-                if cfg!(debug_assertions) {
-                    println!("slow1")
-                } else {
-                    println!("fast1")
+                fn main() {
+                    if cfg!(debug_assertions) {
+                        println!("slow1")
+                    } else {
+                        println!("fast1")
+                    }
+                    bar::baz();
                 }
-                bar::baz();
-            }
-        "#,
+            "#,
         )
         .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
         .file(
             "bar/src/bar.rs",
             r#"
-            pub fn baz() {
-                if cfg!(debug_assertions) {
-                    println!("slow2")
-                } else {
-                    println!("fast2")
+                pub fn baz() {
+                    if cfg!(debug_assertions) {
+                        println!("slow2")
+                    } else {
+                        println!("fast2")
+                    }
                 }
-            }
-        "#,
+            "#,
         )
         .build();
 
@@ -691,14 +780,14 @@ fn run_dylib_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            path = "bar"
-        "#,
+                [dependencies.bar]
+                path = "bar"
+            "#,
         )
         .file(
             "src/main.rs",
@@ -707,15 +796,15 @@ fn run_dylib_dep() {
         .file(
             "bar/Cargo.toml",
             r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
 
-            [lib]
-            name = "bar"
-            crate-type = ["dylib"]
-        "#,
+                [lib]
+                name = "bar"
+                crate-type = ["dylib"]
+            "#,
         )
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
@@ -723,14 +812,204 @@ fn run_dylib_dep() {
     p.cargo("run hello world").run();
 }
 
+#[cargo_test]
+fn run_with_bin_dep() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+
+                [dependencies.bar]
+                path = "bar"
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "0.0.1"
+                authors = []
+
+                [[bin]]
+                name = "bar"
+            "#,
+        )
+        .file("bar/src/main.rs", r#"fn main() { println!("bar"); }"#)
+        .build();
+
+    p.cargo("run")
+        .with_stderr(
+            "\
+[WARNING] foo v0.0.1 ([CWD]) ignoring invalid dependency `bar` which is missing a lib target
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/foo[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
+}
+
+#[cargo_test]
+fn run_with_bin_deps() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.0.1"
+
+                [dependencies.bar1]
+                path = "bar1"
+                [dependencies.bar2]
+                path = "bar2"
+            "#,
+        )
+        .file("src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .file(
+            "bar1/Cargo.toml",
+            r#"
+                [package]
+                name = "bar1"
+                version = "0.0.1"
+                authors = []
+
+                [[bin]]
+                name = "bar1"
+            "#,
+        )
+        .file("bar1/src/main.rs", r#"fn main() { println!("bar1"); }"#)
+        .file(
+            "bar2/Cargo.toml",
+            r#"
+                [package]
+                name = "bar2"
+                version = "0.0.1"
+                authors = []
+
+                [[bin]]
+                name = "bar2"
+            "#,
+        )
+        .file("bar2/src/main.rs", r#"fn main() { println!("bar2"); }"#)
+        .build();
+
+    p.cargo("run")
+        .with_stderr(
+            "\
+[WARNING] foo v0.0.1 ([CWD]) ignoring invalid dependency `bar1` which is missing a lib target
+[WARNING] foo v0.0.1 ([CWD]) ignoring invalid dependency `bar2` which is missing a lib target
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/foo[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
+}
+
+#[cargo_test]
+fn run_with_bin_dep_in_workspace() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [workspace]
+                members = ["foo1", "foo2"]
+            "#,
+        )
+        .file(
+            "foo1/Cargo.toml",
+            r#"
+                [package]
+                name = "foo1"
+                version = "0.0.1"
+
+                [dependencies.bar1]
+                path = "bar1"
+            "#,
+        )
+        .file("foo1/src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .file(
+            "foo1/bar1/Cargo.toml",
+            r#"
+                [package]
+                name = "bar1"
+                version = "0.0.1"
+                authors = []
+
+                [[bin]]
+                name = "bar1"
+            "#,
+        )
+        .file(
+            "foo1/bar1/src/main.rs",
+            r#"fn main() { println!("bar1"); }"#,
+        )
+        .file(
+            "foo2/Cargo.toml",
+            r#"
+                [package]
+                name = "foo2"
+                version = "0.0.1"
+
+                [dependencies.bar2]
+                path = "bar2"
+            "#,
+        )
+        .file("foo2/src/main.rs", r#"fn main() { println!("hello"); }"#)
+        .file(
+            "foo2/bar2/Cargo.toml",
+            r#"
+                [package]
+                name = "bar2"
+                version = "0.0.1"
+                authors = []
+
+                [[bin]]
+                name = "bar2"
+            "#,
+        )
+        .file(
+            "foo2/bar2/src/main.rs",
+            r#"fn main() { println!("bar2"); }"#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] `cargo run` could not determine which binary to run[..]
+available binaries: bar1, bar2, foo1, foo2",
+        )
+        .run();
+
+    p.cargo("run --bin foo1")
+        .with_stderr(
+            "\
+[WARNING] foo1 v0.0.1 ([CWD]/foo1) ignoring invalid dependency `bar1` which is missing a lib target
+[WARNING] foo2 v0.0.1 ([CWD]/foo2) ignoring invalid dependency `bar2` which is missing a lib target
+[COMPILING] foo1 v0.0.1 ([CWD]/foo1)
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[RUNNING] `target/debug/foo1[EXE]`",
+        )
+        .with_stdout("hello")
+        .run();
+}
+
 #[cargo_test]
 fn release_works() {
     let p = project()
         .file(
             "src/main.rs",
             r#"
-            fn main() { if cfg!(debug_assertions) { panic!() } }
-        "#,
+                fn main() { if cfg!(debug_assertions) { panic!() } }
+            "#,
         )
         .build();
 
@@ -746,20 +1025,43 @@ fn release_works() {
     assert!(p.release_bin("foo").is_file());
 }
 
+#[cargo_test]
+fn release_short_works() {
+    let p = project()
+        .file(
+            "src/main.rs",
+            r#"
+                fn main() { if cfg!(debug_assertions) { panic!() } }
+            "#,
+        )
+        .build();
+
+    p.cargo("run -r")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] release [optimized] target(s) in [..]
+[RUNNING] `target/release/foo[EXE]`
+",
+        )
+        .run();
+    assert!(p.release_bin("foo").is_file());
+}
+
 #[cargo_test]
 fn run_bin_different_name() {
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [[bin]]
-            name = "bar"
-        "#,
+                [[bin]]
+                name = "bar"
+            "#,
         )
         .file("src/bar.rs", "fn main() {}")
         .build();
@@ -773,14 +1075,14 @@ fn dashes_are_forwarded() {
         .file(
             "src/bin/bar.rs",
             r#"
-            fn main() {
-                let s: Vec<String> = std::env::args().collect();
-                assert_eq!(s[1], "--");
-                assert_eq!(s[2], "a");
-                assert_eq!(s[3], "--");
-                assert_eq!(s[4], "b");
-            }
-        "#,
+                fn main() {
+                    let s: Vec<String> = std::env::args().collect();
+                    assert_eq!(s[1], "--");
+                    assert_eq!(s[2], "a");
+                    assert_eq!(s[3], "--");
+                    assert_eq!(s[4], "b");
+                }
+            "#,
         )
         .build();
 
@@ -822,22 +1124,22 @@ fn run_with_library_paths() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
-        "#,
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r##"
-            fn main() {{
-                println!(r#"cargo:rustc-link-search=native={}"#);
-                println!(r#"cargo:rustc-link-search={}"#);
-            }}
-        "##,
+                    fn main() {{
+                        println!(r#"cargo:rustc-link-search=native={}"#);
+                        println!(r#"cargo:rustc-link-search={}"#);
+                    }}
+                "##,
                 dir1.display(),
                 dir2.display()
             ),
@@ -846,14 +1148,14 @@ fn run_with_library_paths() {
             "src/main.rs",
             &format!(
                 r##"
-            fn main() {{
-                let search_path = std::env::var_os("{}").unwrap();
-                let paths = std::env::split_paths(&search_path).collect::<Vec<_>>();
-                println!("{{:#?}}", paths);
-                assert!(paths.contains(&r#"{}"#.into()));
-                assert!(paths.contains(&r#"{}"#.into()));
-            }}
-        "##,
+                    fn main() {{
+                        let search_path = std::env::var_os("{}").unwrap();
+                        let paths = std::env::split_paths(&search_path).collect::<Vec<_>>();
+                        println!("{{:#?}}", paths);
+                        assert!(paths.contains(&r#"{}"#.into()));
+                        assert!(paths.contains(&r#"{}"#.into()));
+                    }}
+                "##,
                 dylib_path_envvar(),
                 dir1.display(),
                 dir2.display()
@@ -881,23 +1183,23 @@ fn library_paths_sorted_alphabetically() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            build = "build.rs"
-        "#,
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                build = "build.rs"
+            "#,
         )
         .file(
             "build.rs",
             &format!(
                 r##"
-            fn main() {{
-                println!(r#"cargo:rustc-link-search=native={}"#);
-                println!(r#"cargo:rustc-link-search=native={}"#);
-                println!(r#"cargo:rustc-link-search=native={}"#);
-            }}
-        "##,
+                    fn main() {{
+                        println!(r#"cargo:rustc-link-search=native={}"#);
+                        println!(r#"cargo:rustc-link-search=native={}"#);
+                        println!(r#"cargo:rustc-link-search=native={}"#);
+                    }}
+                "##,
                 dir1.display(),
                 dir2.display(),
                 dir3.display()
@@ -907,15 +1209,15 @@ fn library_paths_sorted_alphabetically() {
             "src/main.rs",
             &format!(
                 r##"
-            fn main() {{
-                let search_path = std::env::var_os("{}").unwrap();
-                let paths = std::env::split_paths(&search_path).collect::<Vec<_>>();
-                // ASCII case-sensitive sort
-                assert_eq!("BBBBBBB", paths[0].file_name().unwrap().to_string_lossy());
-                assert_eq!("aaaaaaa", paths[1].file_name().unwrap().to_string_lossy());
-                assert_eq!("zzzzzzz", paths[2].file_name().unwrap().to_string_lossy());
-            }}
-        "##,
+                    fn main() {{
+                        let search_path = std::env::var_os("{}").unwrap();
+                        let paths = std::env::split_paths(&search_path).collect::<Vec<_>>();
+                        // ASCII case-sensitive sort
+                        assert_eq!("BBBBBBB", paths[0].file_name().unwrap().to_string_lossy());
+                        assert_eq!("aaaaaaa", paths[1].file_name().unwrap().to_string_lossy());
+                        assert_eq!("zzzzzzz", paths[2].file_name().unwrap().to_string_lossy());
+                    }}
+                "##,
                 dylib_path_envvar()
             ),
         )
@@ -944,21 +1246,21 @@ fn run_multiple_packages() {
         .file(
             "foo/Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [workspace]
+                [workspace]
 
-            [dependencies]
-            d1 = { path = "d1" }
-            d2 = { path = "d2" }
-            d3 = { path = "../d3" } # outside of the workspace
+                [dependencies]
+                d1 = { path = "d1" }
+                d2 = { path = "d2" }
+                d3 = { path = "../d3" } # outside of the workspace
 
-            [[bin]]
-            name = "foo"
-        "#,
+                [[bin]]
+                name = "foo"
+            "#,
         )
         .file("foo/src/foo.rs", "fn main() { println!(\"foo\"); }")
         .file("foo/d1/Cargo.toml", &basic_bin_manifest("d1"))
@@ -990,13 +1292,22 @@ fn run_multiple_packages() {
 
     cargo().arg("-p").arg("d1").arg("-p").arg("d2")
                     .with_status(1)
-                    .with_stderr_contains("error: The argument '--package <SPEC>' was provided more than once, but cannot be used multiple times").run();
+                    .with_stderr_contains("error: The argument '--package [<SPEC>...]' was provided more than once, but cannot be used multiple times").run();
 
     cargo()
         .arg("-p")
         .arg("d3")
         .with_status(101)
-        .with_stderr_contains("[ERROR] package `d3` is not a member of the workspace")
+        .with_stderr_contains("[ERROR] package(s) `d3` not found in workspace [..]")
+        .run();
+
+    cargo()
+        .arg("-p")
+        .arg("d*")
+        .with_status(101)
+        .with_stderr_contains(
+            "[ERROR] `cargo run` does not support glob pattern `d*` on package selection",
+        )
         .run();
 }
 
@@ -1006,11 +1317,11 @@ fn explicit_bin_with_args() {
         .file(
             "src/main.rs",
             r#"
-            fn main() {
-                assert_eq!(std::env::args().nth(1).unwrap(), "hello");
-                assert_eq!(std::env::args().nth(2).unwrap(), "world");
-            }
-        "#,
+                fn main() {
+                    assert_eq!(std::env::args().nth(1).unwrap(), "hello");
+                    assert_eq!(std::env::args().nth(2).unwrap(), "world");
+                }
+            "#,
         )
         .build();
 
@@ -1023,9 +1334,9 @@ fn run_workspace() {
         .file(
             "Cargo.toml",
             r#"
-            [workspace]
-            members = ["a", "b"]
-        "#,
+                [workspace]
+                members = ["a", "b"]
+            "#,
         )
         .file("a/Cargo.toml", &basic_bin_manifest("a"))
         .file("a/src/main.rs", r#"fn main() {println!("run-a");}"#)
@@ -1050,18 +1361,18 @@ fn default_run_workspace() {
         .file(
             "Cargo.toml",
             r#"
-            [workspace]
-            members = ["a", "b"]
-        "#,
+                [workspace]
+                members = ["a", "b"]
+            "#,
         )
         .file(
             "a/Cargo.toml",
             r#"
-            [project]
-            name = "a"
-            version = "0.0.1"
-            default-run = "a"
-        "#,
+                [project]
+                name = "a"
+                version = "0.0.1"
+                default-run = "a"
+            "#,
         )
         .file("a/src/main.rs", r#"fn main() {println!("run-a");}"#)
         .file("b/Cargo.toml", &basic_bin_manifest("b"))
@@ -1132,11 +1443,11 @@ fn run_link_system_path_macos() {
             "build.rs",
             &format!(
                 r#"
-            fn main() {{
-                println!("cargo:rustc-link-lib=foo");
-                println!("cargo:rustc-link-search={}");
-            }}
-            "#,
+                fn main() {{
+                    println!("cargo:rustc-link-lib=foo");
+                    println!("cargo:rustc-link-search={}");
+                }}
+                "#,
                 p.target_debug_dir().display()
             ),
         )
