@@ -37,6 +37,38 @@ fn vendor_simple() {
     p.cargo("build").run();
 }
 
+#[cargo_test]
+fn vendor_sample_config() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [dependencies]
+                log = "0.3.5"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    Package::new("log", "0.3.5").publish();
+
+    p.cargo("vendor --respect-source-config")
+        .with_stdout(
+            r#"
+[source.crates-io]
+replace-with = "vendored-sources"
+
+[source.vendored-sources]
+directory = "vendor"
+"#,
+        )
+        .run();
+}
+
 fn add_vendor_config(p: &Project) {
     p.change_file(
         ".cargo/config",
@@ -277,6 +309,72 @@ fn two_lockfiles() {
     p.cargo("build").cwd("bar").run();
 }
 
+#[cargo_test]
+fn test_sync_argument() {
+    let p = project()
+        .no_manifest()
+        .file(
+            "foo/Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [dependencies]
+                bitflags = "=0.7.0"
+            "#,
+        )
+        .file("foo/src/lib.rs", "")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "0.1.0"
+
+                [dependencies]
+                bitflags = "=0.8.0"
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .file(
+            "baz/Cargo.toml",
+            r#"
+                [package]
+                name = "baz"
+                version = "0.1.0"
+
+                [dependencies]
+                bitflags = "=0.8.0"
+            "#,
+        )
+        .file("baz/src/lib.rs", "")
+        .build();
+
+    Package::new("bitflags", "0.7.0").publish();
+    Package::new("bitflags", "0.8.0").publish();
+
+    p.cargo("vendor --respect-source-config --manifest-path foo/Cargo.toml -s bar/Cargo.toml baz/Cargo.toml test_vendor")
+        .with_stderr("\
+error: Found argument 'test_vendor' which wasn't expected, or isn't valid in this context
+
+USAGE:
+    cargo[EXE] vendor [OPTIONS] [path]
+
+For more information try --help",
+        )
+        .with_status(1)
+        .run();
+
+    p.cargo("vendor --respect-source-config --manifest-path foo/Cargo.toml -s bar/Cargo.toml -s baz/Cargo.toml test_vendor")
+        .run();
+
+    let lock = p.read_file("test_vendor/bitflags/Cargo.toml");
+    assert!(lock.contains("version = \"0.8.0\""));
+    let lock = p.read_file("test_vendor/bitflags-0.7.0/Cargo.toml");
+    assert!(lock.contains("version = \"0.7.0\""));
+}
+
 #[cargo_test]
 fn delete_old_crates() {
     let p = project()
@@ -753,3 +851,34 @@ fn vendor_preserves_permissions() {
     let metadata = fs::metadata(p.root().join("vendor/bar/example.sh")).unwrap();
     assert_eq!(metadata.mode() & 0o777, 0o755);
 }
+
+#[cargo_test]
+fn no_remote_dependency_no_vendor() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                [dependencies]
+                bar = { path = "bar" }
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "0.1.0"
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("vendor")
+        .with_stderr("There is no dependency to vendor in this project.")
+        .run();
+    assert!(!p.root().join("vendor").exists());
+}
